/* This include file is a merging of two versions, one for Open VMS and one
 * for UNIX.  An #ifdef was set up with the two version of qfile.h inserted
 * appropriately.
 *                    J. Doggett, 09-SEP-93
 */

/* Modified #ifdef to use __VMS __UNIX, and dummy #else for unsupported
 * systems. Also, added config.h include
 *                    J. Doggett, 07-JAN-94
 */

/******************************************************************************
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/

#include "config.h"

#if defined __VMS
/* ######################################################################### */
/* OpenVMS version ######################################################### */

/* qfile.h   File structure definition for OpenVMS files
 *
 * Programmer: R. White     Date: 16 June 1992
 */
#include <rms.h>
#include <ssdef.h>

struct qfile {
    struct FAB fab;         /* File access block                */
    struct RAB rab;         /* Record access block              */
    char *filename;         /* File name                        */
    int recordsize;         /* Record size (bytes)              */
    int bufsize;            /* Buffer size (bytes)              */
    unsigned char write;    /* Write flag (0 for readonly)      */
    unsigned char crrat;    /* CR attribute flag (0 for no CRs) */
    unsigned char *buffer;  /* Record buffer                    */
    int bptr;               /* Current pointer in buffer        */
    };

#define QFILE struct qfile

/*
 * External procedures
 */
extern QFILE *qcreat();
extern QFILE *qopen();
extern void  qread();
extern void  qwrite();
extern void  fillbuff();
extern void  dumpbuff();
extern int   readint();
extern void  writeint();

/*
 * Macros to get and put characters to files
 * Usage:
 *     c = qgetc(qfile);
 *     qputc(c,qfile);
 */
#define qgetc(f) ((f->bptr<f->bufsize) ? f->buffer[f->bptr++]                  \
                                       : (fillbuff(f), f->buffer[f->bptr++]))

#define qputc(c,f) { if(f->bptr >= f->recordsize) dumpbuff(f);                 \
                     f->buffer[f->bptr++] = c; }

#elif defined __UNIX
/* ######################################################################### */
/* UNIX version ############################################################ */

/* qfile.h   File structure definition
 *
 * Programmer: R. White     Date: 16 June 1992
 */

struct qfile {
    FILE *file;                 /* File descriptor pointer          */
    char *filename;             /* File name                        */
    int recordsize;             /* Record size (bytes)              */
    int bufsize;                /* Buffer size (bytes)              */
    unsigned char write;        /* Write flag (0 for readonly)      */
    unsigned char crrat;        /* CR attribute flag (0 for no CRs) */
    unsigned char *buffer;      /* Record buffer                    */
    int bptr;                   /* Current pointer in buffer        */
    };

#define QFILE struct qfile

/*
 * External procedures
 */
extern QFILE *qcreat();
extern QFILE *qopen();
extern void  qread();
extern void  qwrite();
extern void  fillbuff();
extern void  dumpbuff();
extern int   readint();
extern void  writeint();

/*
 * Macros to get and put characters to files
 * Usage:
 *     c = qgetc(qfile);
 *     qputc(c,qfile);
 */
#define qgetc(f) ((f->bptr<f->bufsize) ? f->buffer[f->bptr++]                  \
                                       : (fillbuff(f), f->buffer[f->bptr++]))

#define qputc(c,f) { if(f->bptr >= f->recordsize) dumpbuff(f);                 \
                     f->buffer[f->bptr++] = c; }

#else
#endif
